--[[-------------------------------------------------------------------------
	InterruptHelper
	Copyright (C) 2011  Morsker
	All Rights Reserved.
	Please contact Morsker through PM on wowace.com.

	Helps with interrupt notifications and announcements.
---------------------------------------------------------------------------]]
local addonName = ...
local InterruptHelper = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceEvent-3.0")
_G.InterruptHelper = InterruptHelper

--------------
--  Locals  --
--------------

-- GLOBALS: _G
-- GLOBALS: InterruptHelperDB
-- GLOBALS: EnableAddOn
-- GLOBALS: LoadAddOn
-- GLOBALS: LibStub
-- GLOBALS: CreateFrame
-- GLOBALS: InterfaceOptionsFrameAddOnsButton1
-- GLOBALS: InterfaceOptionsFrame_OpenToCategory
-- GLOBALS: InterfaceOptions_AddCategory
local GetSpellLink = GetSpellLink
local GetSpellInfo = GetSpellInfo
local GetZonePVPInfo = GetZonePVPInfo
local UnitInBattleground = UnitInBattleground
local UnitInRaid = UnitInRaid
local GetNumPartyMembers = GetNumPartyMembers
local GetChannelName = GetChannelName
local SendChatMessage = SendChatMessage
local select = select
local format = format
local gsub = gsub
local strlower = strlower

local DBVERSION = 1
local db
local cleuFrame = CreateFrame("Frame", "InterruptHelperCLEUFrame"); cleuFrame:Hide();
local castFrame = CreateFrame("Frame", "InterruptHelperCastFrame"); castFrame:Hide();
local playerGUID = UnitGUID("player")
local lastSayFormat
local sayFormatInternal

local defaults = {
	profile = {
		annAnnounce = true,
		annSayFormat = "Interrupted: $S",
		annSolo = "SAY",
		annParty = "PARTY",
		annRaid = "RAID",
		annBattleground = "NONE",
		annCombatZone = "NONE",
	}
}

function InterruptHelper:PushUpvalues()
end

function InterruptHelper:ZoneCheck()
	local pvpType = GetZonePVPInfo()
	self.combatZone = not not (pvpType == "combat")
end

function InterruptHelper:PLAYER_ENTERING_WORLD()
	self:ZoneCheck()
end

function InterruptHelper:SayText(msg)
	local context
	if UnitInBattleground("player") then
		context = "annBattleground"
	elseif UnitInRaid("player") then
		if self.combatZone then
			context = "annCombatZone"
		else
			context = "annRaid"
		end
	elseif GetNumPartyMembers() > 0 then
		context = "annParty"
	else
		context = "annSolo"
	end
	local chatType = db.profile[context]
	if chatType ~= "NONE" then
		local channel
		if chatType == "WHISPER" then
			channel = db.profile[context.."Player"]
		elseif chatType == "CHANNEL" then
			local channelName = strlower(db.profile[context.."Channel"])
			local i,_,iChannelName = 1, GetChannelName(1)
			while iChannelName do
				if strlower(iChannelName) == channelName then
					channel = i
					break
				end
				i,_,iChannelName = i + 1, GetChannelName(i + 1)
			end
			if channel == nil then
				return
			end
		end
		SendChatMessage(msg, chatType, nil, channel)
	end
end

function InterruptHelper.CLEUHandler(cleuFrame, _, timestamp, event, ...)
	if event ~= "SPELL_INTERRUPT" then return end
	local sourceGUID = ...
	if sourceGUID ~= playerGUID then return end

	local spellID = select(10, ...)
	if lastSayFormat ~= db.profile.annSayFormat then
		sayFormatInternal = db.profile.annSayFormat
		sayFormatInternal = gsub(sayFormatInternal, "%%", "%%%%")
		sayFormatInternal = gsub(sayFormatInternal, "$s", "%%1$s")
		sayFormatInternal = gsub(sayFormatInternal, "$S", "%%2$s")
		lastSayFormat = db.profile.annSayFormat
	end
	local spellName = GetSpellInfo(spellID)
	local spellLink = GetSpellLink(spellID)
	local msg = format(sayFormatInternal, spellName, spellLink)
	return InterruptHelper:SayText(msg)
end

function InterruptHelper.CastEventHandler(castFrame, event, unitID)
end

local function LoadOptions()
	if InterruptHelper.options_table then return end
	EnableAddOn("InterruptHelper_Options")
	LoadAddOn("InterruptHelper_Options")
	InterruptHelper:InitOptions()
end

-- GLOBALS: SLASH_INTERRUPTHELPER1
-- GLOBALS: SLASH_INTERRUPTHELPER2
-- GLOBALS: SLASH_INTERRUPTHELPER3
SLASH_INTERRUPTHELPER1 = "/ih"
SLASH_INTERRUPTHELPER2 = "/ihelper"
SLASH_INTERRUPTHELPER3 = "/interrupthelper"
function SlashCmdList.INTERRUPTHELPER()
	LoadOptions()
	LibStub("AceConfigDialog-3.0"):Open("InterruptHelper")
end

function InterruptHelper:OnInitialize()
	self.OnInitialize = nil
	InterruptHelperDB = InterruptHelperDB or { dbversion = DBVERSION }
	db = LibStub("AceDB-3.0"):New("InterruptHelperDB", defaults, "Default")
	InterruptHelper.db = db
	self:PushUpvalues()

	-- UI stub for opening options, but only if we need it.
	if not InterruptHelper.options_table then
		local dummyOptionsFrame = CreateFrame("Frame")
		dummyOptionsFrame.name = "InterruptHelper"
		dummyOptionsFrame:Hide()
		dummyOptionsFrame:SetScript("OnShow", function(frame)
			frame:SetScript("OnShow", nil)
			frame:Hide()
			LoadOptions()
			InterfaceOptionsFrame_OpenToCategory("InterruptHelper")
			local i, button = 1, InterfaceOptionsFrameAddOnsButton1
			while button and button:IsVisible() do
				if button.element.name == "InterruptHelper" and button.element.hasChildren then
					button.toggle:Click()
					return
				end
				i, button = i + 1, _G["InterfaceOptionsFrameAddOnsButton"..(i + 1)]
			end
		end)
		InterfaceOptions_AddCategory(dummyOptionsFrame, "InterruptHelper")
	end

	self.cleuFrame = cleuFrame
	cleuFrame:SetScript("OnEvent", self.CLEUHandler)
	self.castFrame = castFrame
	castFrame:SetScript("OnEvent", self.CastEventHandler)
end

function InterruptHelper:OnEnable()
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneCheck")
	self:ZoneCheck()

	cleuFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

	-- castFrame:RegisterEvent("UNIT_SPELLCAST_START")
	-- castFrame:RegisterEvent("UNIT_SPELLCAST_STOP")
	-- castFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
	-- castFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
	-- castFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE")
	-- castFrame:RegisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE")
	-- castFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
	-- castFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
	-- castFrame:RegisterEvent("PLAYER_FOCUS_CHANGED")
end

function InterruptHelper:OnDisable()
	cleuFrame:UnregisterAllEvents()
	castFrame:UnregisterAllEvents()
end
